package es.abrozas.tfcj2ee.action;


import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

import es.abrozas.tfcj2ee.controller.EntrenamientoController;
import es.abrozas.tfcj2ee.controller.GrupoEntrenamientoController;
import es.abrozas.tfcj2ee.model.Actividad;
import es.abrozas.tfcj2ee.model.Entrenamiento;
import es.abrozas.tfcj2ee.model.GrupoEntrenamiento;
import es.abrozas.tfcj2ee.model.Usuario;

 

public class EntrenamientosAction extends ActionSupport {

	private static final long serialVersionUID = -4679019697243016717L;
	
	private List<Usuario> result;
	
	private Usuario usuario;
    private String mensaje;
    private String descripcion;
    private String fecha;
    private Entrenamiento entrenamiento;
    private List<Entrenamiento> entrenamientos;
    private List<Entrenamiento> planificaciones;
    private List<Usuario> atletas;
    
    private String[] actTipo;
    private String[] actRepeticiones;
    private String[] actTiempo;
    private String[] actDistancia;
    private String[] actDescripcion;
    

    /**
     * Carga los datos necesarios para la home
     * @return SUCCESS en caso de no producirse ningún error, si no devuelve error
     */
    public String loadHome() {   	
    	try {
    		this.usuario = (Usuario) ActionContext.getContext().getSession().get("usuario");
        	this.entrenamientos = EntrenamientoController.getListEntrenamientos(usuario);
    		GrupoEntrenamiento grupoEntrenamiento = GrupoEntrenamientoController.getGrupoByUsuario(usuario);
    		if (grupoEntrenamiento!=null) 
    			this.setPlanificaciones(EntrenamientoController.getListEntrenamientos(grupoEntrenamiento.getEntrenador()));
    		if (usuario.getRol()==1) this.atletas = GrupoEntrenamientoController.getAtletasByGrupo(usuario.getGrupoEntrenamiento());
    		return SUCCESS;
    	} catch (Exception ex) {
    		this.setMensaje(ex.toString());
    		return ERROR;
    	}    	
    }
    
    /**
     * Acción que salva el entrenamiento     
     * @return SUCCESS si no hay ningún problema
     */
    public String saveEntrenamientos() {
    	try {
	    	List <Actividad> actividades = new ArrayList<Actividad>();
	    	int i = 0;
	    	this.usuario = (Usuario) ActionContext.getContext().getSession().get("usuario");
	    	Entrenamiento entrenamiento = new Entrenamiento();
	        entrenamiento.setUsuario(this.usuario);
	        Map<String, Object> map = ActionContext.getContext().getParameters();
	        String[] fecha =  (String[]) map.get("fecha");
	        actTipo = (String[]) map.get("actTipo");
	        actRepeticiones = (String[]) map.get("actRepeticiones");
	        actTiempo = (String[]) map.get("actTiempo");
	        actDistancia = (String[]) map.get("actDistancia");
	        actDescripcion = (String[]) map.get("actDescripcion");
	        entrenamiento.setFecha(fecha[0]);
	        entrenamiento.setDescripcion(((String[]) map.get("descripcion"))[0]);
	    	if (actTipo!=null)    		
	        for (String tipo : actTipo)  {
	        	 Actividad act = new Actividad();
	        	 act.setTipo(Integer.valueOf(tipo));
	        	 act.setDistancia(Float.valueOf(actDistancia[i]));
	        	 long tiempo = Long.parseLong(actTiempo[i].split(":")[0])*360000;
	        	 tiempo += Long.parseLong(actTiempo[i].split(":")[1])*6000;
	        	 tiempo += Long.parseLong(actTiempo[i].split(":")[2].replace(".", ""));
	        	 act.setTiempo(tiempo);
	        	 act.setRepeticiones(Integer.valueOf(actRepeticiones[i]));
	        	 act.setEntrenamiento(entrenamiento);
	        	 act.setDescripcion(actDescripcion[i]);
	        	 actividades.add(act);
		   		 i++;
	         }
	         entrenamiento.setActividades(actividades);
	         EntrenamientoController.saveEntrenamiento(entrenamiento);
	         setMensaje("Entrenamiento a�adido correctamente");
	    	return SUCCESS;
    	} catch (Exception ex) {
    		this.setMensaje(ex.toString());
    		return ERROR;
    	}
    }
    
    /**
     * Acción que muestra un entrenamiento     
     * @return SUCCESS si no hay ningún problema. En caso contrario ERROR
     */
    public String verEntrenamiento() {
        String destino = ERROR; 
        Map<String, Object> map = ActionContext.getContext().getParameters();
        try {
        	int id = Integer.valueOf(((String[]) map.get("id"))[0]);
        	Entrenamiento entrenamiento = EntrenamientoController.getEntrenamientoById(id);
        	List actividades = entrenamiento.getActividades();
        	while (actividades.contains(null)) actividades.remove(null);
        	entrenamiento.setActividades(actividades);
        	this.entrenamiento = entrenamiento;
        	destino = SUCCESS;
         } catch (Exception e) {
                e.printStackTrace();
                setMensaje(e.getMessage());
                return ERROR;
          }
          return destino;
    }
    
    /**
     * Acción que borra un entrenamiento
     * @return Error si no borra el entrenamiento, SUCCESS en caso de hacerlo correctamente
     */
    public String deleteEntrenamiento() {
    	String destino = ERROR;
        Map<String, Object> map = ActionContext.getContext().getParameters();    	
    	try {
        	int id = Integer.valueOf(((String[]) map.get("id"))[0]);
        	Entrenamiento entrenamiento = EntrenamientoController.getEntrenamientoById(id);
    		if (EntrenamientoController.deleteEntrenamiento(entrenamiento)) {
    			destino = SUCCESS;
    			setMensaje("Se ha eliminado el entrenamiento correctamente");
    		}
    	} catch (Exception e) {
            e.printStackTrace();
            setMensaje(e.getMessage());
            return ERROR;    		
    	}
    	return destino;
    }
    
    /**
     * Muestra la creación de un entrenamiento a partir de una planificación
     * @return SUCCESS si tiene éxito. ERROR en caso de existir un problema
     */
    public String entrenoFromPlanificacion() {
    	String destino = ERROR;
        Map<String, Object> map = ActionContext.getContext().getParameters();    	
    	try {
    		int id = Integer.valueOf(((String[]) map.get("id"))[0]);
        	Entrenamiento entrenamiento = EntrenamientoController.getEntrenamientoById(id);
        	List actividades = entrenamiento.getActividades();
        	while (actividades.contains(null)) actividades.remove(null);
        	entrenamiento.setActividades(actividades);
        	this.entrenamiento = entrenamiento;
        	destino = SUCCESS;    		
    	} catch (Exception e) {
            e.printStackTrace();
            setMensaje(e.getMessage());
            return ERROR;    		
    	}
    	return destino;   	
    	
    }
 
/*
 * Getters & Setters		
 */

    /**
     * Devuelve el usuario
     */
    public Usuario getUsuario() {
          return usuario;
    }
    
    /**
     * 
     * @param usuario el usuario a ser establecido
     */
    public void setUsuario(Usuario usuario) {
          this.usuario = usuario;
    }

    /**
     * 
     * @return el listado de usuarios resultado
     */
	public List<Usuario> getResult() {
		return result;
	}

	/**
	 * 
	 * @param mensaje establece el mensaje 
	 */
	public void setMensaje(String mensaje) {
		this.mensaje = mensaje;
	}

	/**
	 * 
	 * @return devuelve el mensaje
	 */
	public String getMensaje() {
		return mensaje;
	}

	/**
	 * 
	 * @param descripcion establece la descripcion
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	/**
	 * 
	 * @return la descripción
	 */
	public String getDescripcion() {
		return descripcion;
	}

	/**
	 * @return the actTiempo
	 */
	public String[] getActTiempo() {
		return actTiempo;
	}

	/**
	 * @param actTiempo the actTiempo to set
	 */
	public void setActTiempo(String[] actTiempo) {
		this.actTiempo = actTiempo;
	}

	/**
	 * @return the actDistancia
	 */
	public String[] getActDistancia() {
		return actDistancia;
	}

	/**
	 * @param actDistancia the actDistancia to set
	 */
	public void setActDistancia(String[] actDistancia) {
		this.actDistancia = actDistancia;
	}

	/**
	 * @return the fecha
	 */
	public String getFecha() {
		return fecha;
	}

	/**
	 * @param fecha the fecha to set
	 */
	public void setFecha(String fecha) {
		this.fecha = fecha;
	}


	/**
	 * @return the actTipo
	 */
	public String[] getActTipo() {
		return actTipo;
	}


	/**
	 * @param actTipo the actTipo to set
	 */
	public void setActTipo(String[] actTipo) {
		this.actTipo = actTipo;
	}


	/**
	 * @return the actRepeticiones
	 */
	public String[] getActRepeticiones() {
		return actRepeticiones;
	}


	/**
	 * @param actRepeticiones the actRepeticiones to set
	 */
	public void setActRepeticiones(String[] actRepeticiones) {
		this.actRepeticiones = actRepeticiones;
	}


	/**
	 * @param actDescripcion the actDescripcion to set
	 */
	public void setActDescripcion(String[] actDescripcion) {
		this.actDescripcion = actDescripcion;
	}


	/**
	 * @return the actDescripcion
	 */
	public String[] getActDescripcion() {
		return actDescripcion;
	}


	/**
	 * @param entrenamiento the entrenamiento to set
	 */
	public void setEntrenamiento(Entrenamiento entrenamiento) {
		this.entrenamiento = entrenamiento;
	}


	/**
	 * @return the entrenamiento
	 */
	public Entrenamiento getEntrenamiento() {
		return entrenamiento;
	}


	/**
	 * @param entrenamientos los entrenamientos a establecer
	 */
	public void setEntrenamientos(List<Entrenamiento> entrenamientos) {
		this.entrenamientos = entrenamientos;
	}


	/**
	 * @return los entrenamientos
	 */
	public List<Entrenamiento> getEntrenamientos() {
		return entrenamientos;
	}

	/**
	 * @param planificaciones el planificaciones a establecer
	 */
	public void setPlanificaciones(List<Entrenamiento> planificaciones) {
		this.planificaciones = planificaciones;
	}

	/**
	 * @return obtiene planificaciones
	 */
	public List<Entrenamiento> getPlanificaciones() {
		return planificaciones;
	}

	/**
	 * @param atletas el atletas a establecer
	 */
	public void setAtletas(List<Usuario> atletas) {
		this.atletas = atletas;
	}

	/**
	 * @return obtiene atletas
	 */
	public List<Usuario> getAtletas() {
		return atletas;
	}
	
	
}
