package es.abrozas.tfcj2ee.action;


import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

import es.abrozas.tfcj2ee.controller.UsuarioController;
import es.abrozas.tfcj2ee.model.Usuario;

 

public class UsuariosAction extends ActionSupport {

	private static final long serialVersionUID = 706612685936410051L;

	private List<Usuario> result;
	private static final String USUARIO_NO_AUTORIZADO    	= "El usuario introducido no está autorizado.";
    private static final String ERROR_INTERNO             	= "Error interno. Por favor, inténtelo otra vez en unos minutos.";
    private static final String USUARIO_NO_REGISTRADO 		= "Error al registrar el usuario";

	private static final String USUARIO_NO_ACTUALIZAD = null;
    private Usuario usuario;
    private String mensaje;
    private String estado;

    /**
     * Acción que carga el perfil del usuario
     * @return SUCCESS o ERROR en caso del éxito de la acción
     */
    public String loadPerfil() {
    	try {
    		this.usuario = (Usuario) ActionContext.getContext().getSession().get("usuario");
    		return SUCCESS;
    	} catch (Exception ex) {
    		this.setMensaje(ex.toString());
    		return ERROR;
    	}
    }

    /**
     * Accion que identifica al usuario en el sistema
     * @return SUCCESS o INPUT en caso del éxito o no de la acción
     */
    public String identificar () {
        String destino = INPUT;
        try {
              if ((usuario=UsuarioController.esUsuarioAutorizado(usuario))!=null) {
                    Map session = ActionContext.getContext().getSession();
                    session.put("usuario", usuario);
                    setEstado(SUCCESS);
                    destino = SUCCESS;
              } else {
                    setMensaje(USUARIO_NO_AUTORIZADO);
                    setEstado(ERROR);
              }
        } catch (Exception e) {
              e.printStackTrace();
              setEstado(ERROR);
              setMensaje(ERROR_INTERNO);
        }
        return destino;
  }

    /**
     * Accion que desconecta al usuario en el sistema
     * @return SUCCESS
     */
    public String logout() {
    	Map session = ActionContext.getContext().getSession();
    	session.clear();
    	return SUCCESS;
    }
    
    /**
     * Acción que registra un nuevo usuario en la base de datos
     * @return SUCCESS o INPUT en caso del éxito o no de la acción
     */
    public String registrar() {
	  this.validateForm();
      String destino = INPUT;
      try {
      	if(UsuarioController.registerUser(usuario)) {
      		System.out.println("success");
      		setMensaje("success");
            setEstado(SUCCESS);
      		destino = SUCCESS;
      	} else {
      		setMensaje(USUARIO_NO_REGISTRADO);
            setEstado(ERROR);
      	}
      } catch (Exception e) {
            e.printStackTrace();
            setEstado(ERROR);
            if (e.getMessage().equals("USUARIO.BBDD.USERNAME_DUPLICADO")) {
            	setMensaje("El nombre de usuario ya existe");
            } else setMensaje(ERROR_INTERNO);
      }
      return destino;
    }
    
    /**
     * Actualiza los datos del usuario
     * @return SUCCESS o ERROR en caso del éxito o no de la acción
     */
    public String actualizar() {
        String destino = ERROR;
        
        try {
        	/* Codificamos apellidos, nombre y ciudad en UTF8 */
    	    String var=((String[]) ActionContext.getContext().getParameters().get("usuario.apellidos"))[0];
    	    byte[] arrByte = var.getBytes("ISO-8859-1");
    	    usuario.setApellidos(new String(arrByte, "UTF-8"));
    	    
    	    var=((String[]) ActionContext.getContext().getParameters().get("usuario.nombre"))[0];
    	    arrByte = var.getBytes("ISO-8859-1");
    	    usuario.setNombre(new String(arrByte, "UTF-8"));
    	    
    	    var=((String[]) ActionContext.getContext().getParameters().get("usuario.ciudad"))[0];
    	    arrByte = var.getBytes("ISO-8859-1");
    	    usuario.setCiudad(new String(arrByte, "UTF-8"));
    	    
        	usuario.setRol(((Usuario) ActionContext.getContext().getSession().get("usuario")).getRol());
        	usuario.setGrupoEntrenamiento(((Usuario) ActionContext.getContext().getSession().get("usuario")).getGrupoEntrenamiento());
	        if (!((String[]) ActionContext.getContext().getParameters().get("usuario.password"))[0].isEmpty())
	        {
	        	if (!((String[]) ActionContext.getContext().getParameters().get("passwordActual"))[0].equals(((Usuario) ActionContext.getContext().getSession().get("usuario")).getPassword())) {
	        		setEstado(ERROR);
	        		setMensaje("La password actual no es correcta");
	        		return ERROR;
	        	} else if(!((String[]) ActionContext.getContext().getParameters().get("usuario.password"))[0].equals( 
	        		((String[]) ActionContext.getContext().getParameters().get("passwordNuevaR"))[0])) {
	        		setEstado(ERROR);
	        		setMensaje("Las passwords nuevas no coinciden");
	        		return ERROR;
	        	} else {
	        		usuario.setPassword(((String[]) ActionContext.getContext().getParameters().get("usuario.password"))[0]);
	        	}
	        	
	        } else usuario.setPassword(((Usuario) ActionContext.getContext().getSession().get("usuario")).getPassword());
        	boolean changeName = true;
        	usuario.setId(((Usuario) ActionContext.getContext().getSession().get("usuario")).getId());
        	if (((String[]) ActionContext.getContext().getParameters().get("usuario.username"))[0].equals(((Usuario) ActionContext.getContext().getSession().get("usuario")).getUsername()))
        		changeName = false;
        	if(UsuarioController.updateUser(usuario,changeName)) {
        		setMensaje("success");
        		setEstado(SUCCESS);
        		destino = SUCCESS;
        	} else {
        		setMensaje(USUARIO_NO_ACTUALIZAD);
              setEstado(ERROR);
        	}
        } catch (Exception e) {
        	  e.printStackTrace();
              setEstado(ERROR);
              if (e.getMessage().equals("USUARIO.BBDD.USERNAME_EXISTS")) {
              	setMensaje("El nombre de usuario ya existe");
              } else setMensaje(ERROR_INTERNO);
              return ERROR;
        }
        Map session = ActionContext.getContext().getSession();
        session.put("usuario", usuario);
        return destino;
  }
    
    
    /**
     * Acción que envía la contraseña al correo electrónico del propietario
     * @return SUCCESS o ERROR en caso del éxito o no de la acción
     */
    public String recuperarPassword() {
    	try {
    		Usuario userDB = null;
    		if ((userDB = UsuarioController.getUsuarioByNameEmail(usuario))== null) { 
    			setEstado(ERROR);
    			setMensaje("No se ha encontrado el usuario con esos datos");
    			return ERROR;
    		}
    		String host = "127.0.0.1";
            String from = "test@localhost";
            String to = userDB.getEmail();
            
            // Get system properties
            Properties properties = System.getProperties();

            // Setup mail server
           properties.setProperty("mail.smtp.host", host);

            // Get the default Session object.
            Session session = Session.getDefaultInstance(properties);

            // Create a default MimeMessage object.
            MimeMessage message = new MimeMessage(session);

            // Set the RFC 822 "From" header field using the
            // value of the InternetAddress.getLocalAddress method.
            message.setFrom(new InternetAddress(from));

            // Add the given addresses to the specified recipient type.
            message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));

            // Set the "Subject" header field.
            message.setSubject("Recuperación de contraseña");
            
            // Sets the given String as this part's content,
            // with a MIME type of "text/plain".
            message.setText("Se ha solicitado la recuperación de contraseña de elMister. \r\n Su contraseña es" + userDB.getPassword());

            // Send message
            Transport.send(message);    		
    	} catch (Exception e) {
    		e.printStackTrace();
    		setEstado(ERROR);
    		setMensaje(e.getMessage());    		
    	}
    	
    	return SUCCESS;
    }
    
    /**
     * Validación del formulario de registro
     */
    public void validateForm(){			
		if ((usuario == null) || (usuario.getPassword() == null) || (usuario.getPassword().length() == 0 )){	
				addFieldError( "usuario.password", "La contraseña es obligatoria." );
			}			
		if ((usuario == null) || ( usuario.getRol() == -1 )){	
				addFieldError( "usuario.rol", "El rol es requerido." );
			}			
		if ((usuario == null) || (usuario.getUsername() == null) || ( usuario.getUsername().length() == 0 )){	
				addFieldError( "usuario.username", "El nombre de usuario es obligatorio" );
			}
	}
		
	/**
	 * @return the usuario
	 */
    public Usuario getUsuario() {
          return usuario;
    }
    
	/**
	 * @param usuario the usuario to set
	 */
    public void setUsuario(Usuario usuario) {
          this.usuario = usuario;
    }
    
	/**
	 * @return the result
	 */
	public List<Usuario> getResult() {
		return result;
	}

	/**
	 * @param mensaje the mensaje to set
	 */
	public void setMensaje(String mensaje) {
		this.mensaje = mensaje;
	}

	/**
	 * @return the mensaje
	 */
	public String getMensaje() {
		return mensaje;
	}

	/**
	 * @param estado the estado to set
	 */
	public void setEstado(String estado) {
		this.estado = estado;
	}

	/**
	 * @return the estado
	 */
	public String getEstado() {
		return estado;
	}
}
