package es.abrozas.tfcj2ee.controller;

import java.util.List;

import es.abrozas.tfcj2ee.dao.EntrenamientoDAO;
import es.abrozas.tfcj2ee.model.Entrenamiento;
import es.abrozas.tfcj2ee.model.Usuario;

public class EntrenamientoController {

	/**
	 * Devuelve el listado de entrenamiento de un usuario determinado
	 * @param user El usuario sobre el que se desea obtener los entrenamientos
	 * @return el listado de entrenamientos
	 * @throws Exception en caso de producirse un error al recuperar los entrenamientos
	 */
	public static List<Entrenamiento> getListEntrenamientos(Usuario user) throws Exception {
		EntrenamientoDAO entrenamientoDAO = new EntrenamientoDAO();
		List<Entrenamiento> entrenamientos = null;
		try {
			entrenamientos = entrenamientoDAO.getListEntrenamientos(user);				
		} catch (Exception e) {
			throw e;
		}
		return entrenamientos;
	}
	
	/**
	 * Devuelve el entrenamiento correspondiente a un identificador determinado 
	 * @param id el identificador del entrenamiento
     * @return el entrenamiento
     * @throws Exception cuando existe un error en la recuperación del entrenamiento
	 */
	public static Entrenamiento getEntrenamientoById(int id) throws Exception {
		EntrenamientoDAO entrenamientoDAO = new EntrenamientoDAO();
		Entrenamiento entrenamiento = null;
		try {
			entrenamiento = entrenamientoDAO.getEntrenamientoById(id);
		} catch (Exception e) {
			throw e;
		}
		return entrenamiento;
	}
	
	/**
	 * Añade un nuevo entrenamiento
	 * @param entrenamiento  el entrenamiento que se desea registrar
	 * @return true o false en función del éxito de la acción
	 * @throws Exception en caso de producirse un error en el salvado
	 */
	public static boolean addEntrenamiento(Entrenamiento entrenamiento) throws Exception {
		EntrenamientoDAO entrenamientoDAO = new EntrenamientoDAO();
        boolean registrado = false;
        System.out.println("registerEntrenamiento");
        try {
        	if (entrenamientoDAO.createEntrenamiento(entrenamiento) != null ) {
        		registrado = true;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
	}

	/**
	 * Salva un entrenamiento
	 * @param entrenamiento el entrenamiento a ser salvado
	 * @return true o false en función del éxito de la acción
	 * @throws Exception en caso de producirse un error al salvar el entrenamiento
	 */
	public static boolean saveEntrenamiento(Entrenamiento entrenamiento) throws Exception {
		EntrenamientoDAO entrenamientoDAO = new EntrenamientoDAO();
        boolean registrado = false;
        System.out.println("registerEntrenamiento");
        try {
        	if (entrenamientoDAO.createEntrenamiento(entrenamiento) != null ) {
        		registrado = true;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
		
	}
	
	/**
	 * Elimina un entrenamiento determinado
	 * @param entrenamiento el entrenamiento a ser eliminado
	 * @return true o false en fucnión del éxito o no de la acción
	 * @throws Exception si se produce un erro en el borrado del entrenamiento
	 */
	public static boolean deleteEntrenamiento(Entrenamiento entrenamiento) throws Exception {
		EntrenamientoDAO entrenamientoDAO = new EntrenamientoDAO();
		boolean eliminado = entrenamientoDAO.deleteEntrenamiento(entrenamiento);
		return eliminado;
	}

}