package es.abrozas.tfcj2ee.controller;

import java.util.List;

import es.abrozas.tfcj2ee.dao.GrupoEntrenamientoDAO;
import es.abrozas.tfcj2ee.dao.UsuarioDAO;
import es.abrozas.tfcj2ee.model.Usuario;
import es.abrozas.tfcj2ee.model.GrupoEntrenamiento;

public class GrupoEntrenamientoController {

	/**
	 * Devuelve el grupo de entrenamiento de un usuario 
	 * @param usuario el usuario del que se quiere conocer el grupo de entrenamiento
	 * @return el grupo de entrenamiento del usuario
	 * @throws Exception en caso de producirse un error
	 */
	public static GrupoEntrenamiento getGrupoByUsuario(Usuario usuario) throws Exception {
		GrupoEntrenamientoDAO grupoEntrenamientoDAO = new GrupoEntrenamientoDAO();
		GrupoEntrenamiento grupoEntrenamiento = null;
		try {
			grupoEntrenamiento = grupoEntrenamientoDAO.getGrupoByUsuario(usuario);
		} catch (Exception e) {
			throw e;
		}		
		return grupoEntrenamiento;
	}
	
	/**
	 * Recuperas un grupo de entrenamiento a partir de su identificador
	 * @param id el identificador del grupo
	 * @return el grupo de entrenamiento
	 * @throws Exception en caso de producirse un error
	 */
	public static GrupoEntrenamiento getGrupoById(int id) throws Exception {
		GrupoEntrenamientoDAO grupoEntrenamientoDAO = new GrupoEntrenamientoDAO();
		GrupoEntrenamiento grupo = null;
		try {
			grupo = grupoEntrenamientoDAO.getGrupoById(id);
		} catch (Exception e) {
			throw e;
		}
		return grupo;
	}

	/**
	 * Recuperas el listado de atletas de un grupo de entrenamiento
	 * @param grupoEntrenamiento el grupo de entrenamiento sobre el que se desea hacer la consulta
	 * @return el lista de usuarios
	 * @throws Exception en caso de producirse un error
	 */
	public static List<Usuario> getAtletasByGrupo(
			GrupoEntrenamiento grupoEntrenamiento) throws Exception {
		if (grupoEntrenamiento==null) return null;
		UsuarioDAO grupoEntrenamientoDAO = new UsuarioDAO();
		List<Usuario> usuarios = null;
		try {
			usuarios = grupoEntrenamientoDAO.getAtletasByGrupo(grupoEntrenamiento);
		} catch (Exception e) {
			throw e;
		}		
		return usuarios;
	}

	/**
	 * Crea un grupo de entrenamiento
	 * @param grupoEntrenamiento el grupo de entrenamiento a crear
	 * @return el grupo de entrenamiento creado, con su nuevo identificador único
	 * @throws Exception en caso de producirse un error
	 */
	public static GrupoEntrenamiento crearGrupo(
			GrupoEntrenamiento grupoEntrenamiento) throws Exception {
		if (grupoEntrenamiento==null) return null;
		GrupoEntrenamiento grupo = null;
		try {
			GrupoEntrenamientoDAO grupoEntrenamientoDAO = new GrupoEntrenamientoDAO();
			grupo = grupoEntrenamientoDAO.createGrupo(grupoEntrenamiento);
		} catch (Exception e) {
			throw e;
		}
		return grupo;
	}

	/**
	 * Recuperamos el listado de los grupos de entrenamiento existentes
	 * @return el listado de los grupos de entrenamiento
	 * @throws Exception en caso de producirse un error
	 */
	public static List<GrupoEntrenamiento> getGrupos() throws Exception {
		GrupoEntrenamientoDAO grupoEntrenamientoDAO = new GrupoEntrenamientoDAO();
		List<GrupoEntrenamiento> grupos = null;
		try {
			grupos = grupoEntrenamientoDAO.getListGrupos();
		} catch (Exception e) {
			throw e;
		}		
		return grupos;
	}
	


}