package es.abrozas.tfcj2ee.controller;

import es.abrozas.tfcj2ee.dao.UsuarioDAO;
import es.abrozas.tfcj2ee.model.Usuario;

public class UsuarioController {

	/**
	 * Devuelve el usuario en caso de estar registrado en la ddbb con el usuario y contraseña correctos
	 * @param usuario el usuario a buscar
	 * @return el usuario
	 * @throws Exception en caso de producirse un error con la recuperación de los datos
	 */
	public static Usuario esUsuarioAutorizado (Usuario usuario) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO ();
        Usuario result = null;
        try {
        	result = usuarioDAO.getUsuarioByUsernamePass(usuario);        	
        } catch (Exception e) {
        	throw e;
        }
        return result;
	}
	
	/**
	 * Registra un nuevo usuario 	
	 * @param usuario el usuario a registrar
	 * @return true o false en función del éxito de la acción
	 * @throws Exception en caso de producirse un error
	 */
	public static boolean registerUser(Usuario usuario) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO();
        boolean registrado = false;
        try {
        	if (usuarioDAO.createUsuario(usuario) != null ) {
        		registrado = true;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
	}
	
	/**
	 * Salva el usuario en la base de datos
	 * @param usuario el usuario a salvar
	 * @return true o false en caso del éxito o no de la acción
	 * @throws Exception en caso de producirse un error
	 */
	public static boolean saveUser(Usuario usuario) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO();
        boolean registrado = false;
        try {
        	if (usuarioDAO.saveUsuario(usuario) != null ) {
        		registrado = true;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
	}

	/**
	 * Salva el usuario, teniendo en cuenta si se cambia el username o no
	 * @param usuario el usuario a salvar
	 * @param changeName si se cambia el nombre de usuario o no
	 * @return true o false en función del éxito de la acción
	 * @throws Exception en caso de producirse un error
	 */
	public static boolean updateUser(Usuario usuario, boolean changeName) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO();
        boolean registrado = false;
        try {
        	if (usuarioDAO.updateUsuario(usuario,changeName) != null ) {
        		registrado = true;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
	}
	
	/**
	 * Salva el usuario, teniendo en cuenta si se cambia el username o no y devuelve el usuario salvado
	 * @param usuario el usuario a salvar
	 * @param changeName si se cambia el nombre de usuario o no
	 * @return el usuario salvado
	 * @throws Exception en caso de producirse un error
	 */
	public static Usuario getUpdatedUser(Usuario usuario, boolean changeName) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO();
        Usuario registrado = null;
        try {
        	if ((registrado = usuarioDAO.updateUsuario(usuario,changeName) )!= null ) {
        		return registrado;
        	}
        } catch (Exception e) {
        	throw e;
        }
        return registrado;
	}

	/**
	 * Devuelve el usuario a partir del nombre de usuario y email
	 * @param usuario el usuario a buscar (con su username y email)
	 * @return el Usuario encontrado o null en caso de no encontrarlo
	 * @throws Exception en caso de producirse un error
	 */
	public static Usuario getUsuarioByNameEmail(Usuario usuario) throws Exception {
		UsuarioDAO usuarioDAO = new UsuarioDAO ();
        Usuario result = null;
        try {
        	result = usuarioDAO.getUsuarioByUsernameEmail(usuario);
        	
        } catch (Exception e) {
        	throw e;
        }
        return result;
	}

}