package es.abrozas.tfcj2ee.dao;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import es.abrozas.tfcj2ee.model.Entrenamiento;
import es.abrozas.tfcj2ee.model.Usuario;
import es.abrozas.tfcj2ee.utils.HibernateUtil;


public class EntrenamientoDAO {
		
	/**
	 * Devuelve el listado de entrenamientos a partir de un usuario
	 * @return el listado de entrenamientos
	 * @throws Exception Cuando hay un problema con la recuperación de los datos
	 */
      public List<Entrenamiento> getListEntrenamientos(Usuario user) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  List<Entrenamiento> result = null;
    	  
    	  try {
	          tx = session.beginTransaction();
	          Query hQuery = session.createQuery("from Entrenamiento as e where e.usuario= :usuario_id order by fecha desc");
	          hQuery.setLong("usuario_id", user.getId());
	          result=(List<Entrenamiento>)hQuery.list();
	          session.getTransaction().commit();	          
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }
    	  
    	  return result;          
      }

      /**
       * Crea un entrenamiento en la bbdd y devuelve el entrenamiento con su identificador actualizado
       * @param entrenamiento el entrenamiento a crear
       * @return el entrenamiento creado
       * @throws Exception En caso de producirse un error en la creación del entrenamiento
       */
      public Entrenamiento createEntrenamiento(Entrenamiento entrenamiento) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Entrenamiento result = null;

    	  try {
	          tx = session.beginTransaction();
	          Long entrenamientoId = (Long)session.save("Entrenamiento",entrenamiento);
	          entrenamiento.setId(entrenamientoId);
	          session.getTransaction().commit();  
	          result = entrenamiento;       	          
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }
    	  
    	  return result;
    	  
    	  
      }

      /**
       * Recupera un entrenamiento de la bbdd a partir de su identificador único
       * @param id el identificador del entrenamiento
       * @return el entrenamiento
       * @throws Exception cuando existe un error en la recuperación del entrenamiento
       */
	public Entrenamiento getEntrenamientoById(int id) throws Exception {
		HibernateUtil.createSessionFactory();
		Session session = HibernateUtil.getSession();
		Transaction tx = null;
		Entrenamiento result = null;

		try {
	    	  tx = session.beginTransaction();
	    	  List<Entrenamiento> entrenamientoList;
	    	  Query hQuery =
	    		    session.createQuery("from Entrenamiento as e where e.id= :id");
	    	  hQuery.setLong("id", id);
	    	  
	    	  entrenamientoList = hQuery.list();
	    	  tx.commit();
	    	  if (entrenamientoList.size()>1) throw (new Exception("ENTRENAMIENTO.BBDD.ID_DUPLICADO"));
	    	  if (entrenamientoList.size()==0) throw (new Exception("ENTRENAMIENTO.BBDD.ID_NO_ENCONTRADO"));
	    	  
	    	  result = (Entrenamiento) entrenamientoList.get(0);
  	  	}  catch (RuntimeException e) {
  	  		if(tx != null) tx.rollback();
  	  		throw e;
		} finally {
			session.close();
		}
  
		return result;
		
	}
	
	/**
	 * Borra un entrenamiento de la base de datos y devuelve true en caso de que tenga éxito
	 * @param entrenamiento El objeto del entrenamiento a borrar
	 * @return el éxito o no de la acción
	 * @throws Exception cuando se produce un error en el borrado
	 */
	public boolean deleteEntrenamiento(Entrenamiento entrenamiento) throws Exception {
		HibernateUtil.createSessionFactory();
		Session session = HibernateUtil.getSession();
		Transaction tx = null;
		boolean result = false;

		try {
			  tx = session.beginTransaction();
	    	  session.delete(entrenamiento);
	    	  tx.commit();
	    	  result = true;
		}catch (Exception e) {
  	  		if(tx != null) tx.rollback();
  	  		throw e;
		} finally {
			session.close();
		}
		return result;
		
	}
      
     
}
