package es.abrozas.tfcj2ee.dao;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import es.abrozas.tfcj2ee.model.Usuario;
import es.abrozas.tfcj2ee.model.GrupoEntrenamiento;
import es.abrozas.tfcj2ee.utils.HibernateUtil;


public class GrupoEntrenamientoDAO {

	/**
	 * Obtiene el grupo de entrenamiento registrado a un usuario
	 * @param user el usuario sobre el cual se quiere conocer el grupo de entrenamiento
	 * @return el grupo de entrenamiento
	 * @throws Exception cuando existe un problema en la recuperación de los datos
	 */
	public GrupoEntrenamiento getGrupoByUsuario(Usuario user) throws Exception {
	 	if (user.getGrupoEntrenamiento()==null) return null;
		HibernateUtil.createSessionFactory();
	 	Session session = HibernateUtil.getSession();
	 	Transaction tx = null;
	 	GrupoEntrenamiento result = null;
	 	
	 	try {
	 		tx = session.beginTransaction();
	 		Query hQuery = session.createQuery("from GrupoEntrenamiento as g where g.id= :id");
	 		hQuery.setLong("id", user.getGrupoEntrenamiento().getId());
	 		System.out.println("ID del grupo: "+ String.valueOf(user.getGrupoEntrenamiento().getId()));
	 		List<GrupoEntrenamiento> gruposList = (List<GrupoEntrenamiento>) hQuery.list();
	    	tx.commit();
	    	if (gruposList.size()>1) throw (new Exception("GRUPOSENTRENAMIENTO.BBDD.ID_DUPLICADO"));
	    	if (gruposList.size()==0) throw (new Exception("GRUPOSENTRENAMIENTO.BBDD.ID_NO_ENCONTRADO"));
	    	result = (GrupoEntrenamiento) gruposList.get(0);
	    		          
	 	} catch (RuntimeException e) {
	 		if(tx != null) tx.rollback();
	 		throw e;
	 	} finally {
	 		session.close();
	 	}
  
	 	return result;          

		}

	/**
	 * Crea un grupo de entrenamiento nuevo en la base de datos
	 * @param grupoEntrenamiento el grupo de entrenamiento a almacenar
	 * @return el grupo de entrenamiento con su identificador único actualizado
	 */
	public GrupoEntrenamiento createGrupo(GrupoEntrenamiento grupoEntrenamiento) {
		  HibernateUtil.createSessionFactory();
		  Session session = HibernateUtil.getSession();
		  Transaction tx = null;
		  GrupoEntrenamiento result = null;
		
		  try {
		        tx = session.beginTransaction();
		        long id = (Long)session.save("GrupoEntrenamiento",grupoEntrenamiento);
		        grupoEntrenamiento.setId(id);
		        session.getTransaction().commit();  
		        result = grupoEntrenamiento;	          	          
		  } catch (RuntimeException e) {
		    	  if(tx != null) tx.rollback();
		    	  throw e;
		  } finally {
			  session.close();
		  }    	  
		  return result;
	}

	/**
	 * Devuelve el listado de los grupos de entrenamiento existentes en la base de datos
	 * @return el listado de grupos de entrenamiento
	 */
	public List<GrupoEntrenamiento> getListGrupos() {
		HibernateUtil.createSessionFactory();
  	  Session session = HibernateUtil.getSession();
  	  Transaction tx = null;
  	  List<GrupoEntrenamiento> result = null;
  	  
  	  try {
	          tx = session.beginTransaction();
	          Query hQuery = session.createQuery("from GrupoEntrenamiento as g");
	          result=(List<GrupoEntrenamiento>)hQuery.list();
	          session.getTransaction().commit();	          
  	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
  	  } finally {
  		  session.close();
  	  }
  	  
  	  return result;    
	}

	/**
	 * Devuelve el grupo de entrenamiento registrado en la base de datos con un identificador expecífico
	 * @param id El identificador del grupo de entrenamiento
	 * @return el grupo de entrenamiento
	 * @throws Exception en caso de producirse un error en la recuperación del grupo 
	 */
	public GrupoEntrenamiento getGrupoById(int id) throws Exception {
		HibernateUtil.createSessionFactory();
	 	Session session = HibernateUtil.getSession();
	 	Transaction tx = null;
	 	GrupoEntrenamiento result = null;
	 	
	 	try {
	 		tx = session.beginTransaction();
	 		Query hQuery = session.createQuery("from GrupoEntrenamiento as g where g.id= :id");
	 		hQuery.setLong("id", id);
	 		List<GrupoEntrenamiento> gruposList = (List<GrupoEntrenamiento>) hQuery.list();
	    	tx.commit();
	    	if (gruposList.size()>1) throw (new Exception("GRUPOSENTRENAMIENTO.BBDD.ID_DUPLICADO"));
	    	if (gruposList.size()==0) throw (new Exception("GRUPOSENTRENAMIENTO.BBDD.ID_NO_ENCONTRADO"));
	    	result = (GrupoEntrenamiento) gruposList.get(0);
	    		          
	 	} catch (RuntimeException e) {
	 		if(tx != null) tx.rollback();
	 		throw e;
	 	} finally {
	 		session.close();
	 	}
  
	 	return result;   
	}
}
