package es.abrozas.tfcj2ee.dao;

import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import es.abrozas.tfcj2ee.model.Usuario;
import es.abrozas.tfcj2ee.model.GrupoEntrenamiento;
import es.abrozas.tfcj2ee.utils.HibernateUtil;


public class UsuarioDAO {

      /**
       * Procedimiento que utilizaremos para recuperar un usuario de la bbdd mediante su nombre de usuario.
       * Hemos de tener en cuenta que el nombre de usuario es unico, por lo que solo es posible un resultado por consulta.
       * @param usuario
       * @return
       * @throws Exception
       */
      public Usuario getUsuarioByUsernamePass (Usuario usuario) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Boolean find = false;
    	  try
    	  {
	    	  tx = session.beginTransaction();
	    	  List<Usuario> userList;
	    	  Query hQuery =
	    		    session.createQuery("from Usuario as u where u.username= :username");
	    	  hQuery.setString("username", usuario.getUsername());
	    	  
	    	  userList = hQuery.list();
	    	  tx.commit();
	    	  if (userList.size()>1) throw (new Exception("USUARIO.BBDD.USERNAME_DUPLICADO"));
			  Iterator<Usuario> userIt = userList.iterator();
	    	  while (userIt.hasNext())    		  
		  		{	
	    		   Usuario user = (Usuario)userIt.next();
	    		   if (user.getPassword().equals(usuario.getPassword())) {
		  			   	find = true;
		  			   	usuario = user; 
		  			   	break;
		  		   }
		  		}
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }
    	  Usuario resultado = null;
    	  if (find) { 
    		  resultado = usuario;
    	  }
          return resultado;

      }
      
      /**
       * Devuelve un usuario por su nombre usuario. En caso de no encontrarlo devuelve null
       * @param usuario el usuario que desea buscar
       * @return El usuario encontrado. En caso de no encontrarlo devuelve null
       * @throws Exception en caso de producirse un error en la recuperación de datos
       */
      public Usuario getUsuarioByUsername (Usuario usuario) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Boolean find = false;
    	  try
    	  {
	    	  tx = session.beginTransaction();
	    	  List<Usuario> userList;
	    	  Query hQuery =
	    		    session.createQuery("from Usuario as u where u.username= :username");
	    	  hQuery.setString("username", usuario.getUsername());
	    	  
	    	  userList = hQuery.list();
	    	  tx.commit();
	    	  if (userList.size()>1) throw (new Exception("USUARIO.BBDD.USERNAME_DUPLICADO"));
	    	  if (userList.size()==0) return null;
			  Iterator<Usuario> userIt = userList.iterator();
			  usuario = (Usuario) userList.get(0);
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }
    	  Usuario resultado =  usuario;
    	  return resultado;

      }
      
      /**
       * Crea un usuario nuevo a partir del usuario pasado como parámetro
       * @param usuario el usuario que se desea crear en la ddbb
       * @return el usuario generado 
       * @throws Exception en caso de producirse un error en la creación del usuario
       */
      public Usuario createUsuario(Usuario usuario) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Usuario result = null;

    	  try {
	          tx = session.beginTransaction();
	          if (this.getUsuarioByUsername(usuario)==null) {
	        	  Long userId = (Long)session.save("Usuario",usuario);
	        	  usuario.setId(userId);
		          session.getTransaction().commit();  
		          result = usuario;
	          } else throw(new Exception("USUARIO.BBDD.USERNAME_EXISTS"));	          	          
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }    	  
    	  return result;
      }

      
      /**
       * Salva el usuario pasado por parámetro y devuelve el objeto salvado
       * @param usuario el objeto usuario que se desea salvar en la base de datos
       * @return el usuario salvado
       * @throws Exception cuando se produce un error
       */
      public Usuario saveUsuario(Usuario usuario) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Usuario result = null;

    	  try {
	          tx = session.beginTransaction();
	          if (this.getUsuarioByUsername(usuario)!=null) {
	        	  Long userId = (Long)session.save("Usuario",usuario);
	        	  usuario.setId(userId);
		          session.getTransaction().commit();  
		          result = usuario;
	          } else throw(new Exception("USUARIO.BBDD.USERNAME_NO_EXISTE"));	          	          
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }    	  
    	  return result;
      }
      
      /**
       * Actualiza el usuario pasado por parámetro y devuelve el objeto salvado
       * @param usuario el objeto usuario que se desea actualizar en la base de datos
       * @param changeName indica si se está tratando de cambiar el nombre de usuario
       * @return el usuario actualizado
       * @throws Exception cuando se produce un error
       */
      public Usuario updateUsuario(Usuario usuario, boolean changeName) throws Exception {
    	  HibernateUtil.createSessionFactory();
    	  Session session = HibernateUtil.getSession();
    	  Transaction tx = null;
    	  Usuario result = null;

    	  try {
	          tx = session.beginTransaction();
	          if (changeName) {
	        	  if (this.getUsuarioByUsername(usuario)!=null)
	        		  throw(new Exception("USUARIO.BBDD.USERNAME_EXISTS"));
	          }
	          if (this.getUsuarioById(usuario)!=null) {
	        	  session.update("Usuario",usuario);
		          session.getTransaction().commit();  
		          result = usuario;
	          } else throw(new Exception("USUARIO.BBDD.USERNAME_NO_EXISTE"));	          	          
    	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
    	  } finally {
    		  session.close();
    	  }    	  
    	  return result;
      }

      /**
       * Obtiene un usuario a partir de su identificador
       * @param usuario el usuario que desea buscar (sólo tiene en cuenta su id)
       * @return el usuario encontrado. Null en caso de no encontrar el usuario
       * @throws Exception en caso de producirse un error
       */
	private Object getUsuarioById(Usuario usuario) throws Exception {
  	  HibernateUtil.createSessionFactory();
	  Session session = HibernateUtil.getSession();
	  Transaction tx = null;
	  Boolean find = false;
	  Usuario resultado = null;
		
	  try
  	  {
	    	  tx = session.beginTransaction();
	    	  List<Usuario> userList;
	    	  Query hQuery =
	    		  
	    		    session.createQuery("from Usuario as u where u.id= :id");
	    	  hQuery.setLong("id", usuario.getId());
	    	  
	    	  userList = hQuery.list();
	    	  tx.commit();
	    	  if (userList.size()>1) throw (new Exception("USUARIO.BBDD.ID_DUPLICADO"));
	    	  if (userList.size()==0) throw (new Exception("USUARIO.BBDD.ID_NO_ENCONTRADO"));
	    	  
	    	  resultado = (Usuario) userList.get(0);
  	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
  	  } finally {
  		  session.close();
  	  }
  	  
        return resultado;
	}

	/**
	 * Devuelve el listado de atletas que pertenecen a un grupo de entrenamiento determinado
	 * @param grupoEntrenamiento El grupo de entrenamiento sobre el que se desean buscar miembros
	 * @return el listado de usuarios del grupo de entrenamiento
	 */
	public List<Usuario> getAtletasByGrupo(
	  GrupoEntrenamiento grupoEntrenamiento) {
	  HibernateUtil.createSessionFactory();
   	  Session session = HibernateUtil.getSession();
   	  Transaction tx = null;
   	  List<Usuario> result = null;
   	  
   	  try {
	          tx = session.beginTransaction();
	          Query hQuery =
	    		    session.createQuery("from Usuario as u where u.grupoEntrenamiento= :grupo_id AND u.id != :entrenador_id");
	    	  hQuery.setLong("grupo_id", grupoEntrenamiento.getId());
	    	  hQuery.setLong("entrenador_id", grupoEntrenamiento.getEntrenador().getId());
	    	  result = (List<Usuario>)hQuery.list();;
	          session.getTransaction().commit();	          
   	  } catch (RuntimeException e) {
	    	  if(tx != null) tx.rollback();
	    	  throw e;
   	  } finally {
   		  session.close();
   	  }
   	  
   	  return result;  
	}


	/**
	 * Devuelve un usuario a partir de su nombre de usuario y email
	 * @param usuario el usuario que desea buscar (tiene en cuento los campos username y email
	 * @return el usuario encontrado o null en caso de no encontrarlo
	 * @throws Exception en caso de producirse un error
	 */
	public Usuario getUsuarioByUsernameEmail(Usuario usuario) throws Exception {
		  HibernateUtil.createSessionFactory();
	  	  Session session = HibernateUtil.getSession();
	  	  Transaction tx = null;
	  	  Boolean find = false;
	  	  Usuario resultado = null;
	  	  try
	  	  {
		    	  tx = session.beginTransaction();
		    	  List<Usuario> userList;
		    	  Query hQuery =
		    		    session.createQuery("from Usuario as u where u.username= :username AND u.email = :email");
		    	  hQuery.setString("username", usuario.getUsername());
		    	  hQuery.setString("email", usuario.getEmail());
		    	  userList = hQuery.list();
		    	  tx.commit();
		    	  if (userList.size()>1) throw (new Exception("USUARIO.BBDD.USERNAME_DUPLICADO"));
		    	  if (userList.size()==0) return null;
		    	  
		    	  resultado = (Usuario) userList.get(0);
	  	  } catch (RuntimeException e) {
		    	  if(tx != null) tx.rollback();
		    	  throw e;
	  	  } finally {
	  		  session.close();
	  	  }
	  	  return resultado;
	}
      
     
}
