package es.abrozas.tfcj2ee.model;

import java.io.Serializable;
import java.sql.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import es.abrozas.tfcj2ee.model.Usuario;

public class Entrenamiento implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8116135767078158124L;
	private long id;
	private Date fecha;
	private String descripcion;
	private Usuario usuario;

	private List<Actividad> actividades;
	
	public Entrenamiento() {
		
	}

	/**
	 * 
	 * @param usuario Establece el usuario que realiza el entrenamiento
	 */
	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	/**
	 * 
	 * @return el usuario que realiza el entrenamiento
	 */
	public Usuario getUsuario() {
		return usuario;
	}

	/**
	 * 
	 * @param fecha Establece la fecha del entrenamiento
	 */
	public void setFecha(Date fecha) {
		this.fecha = fecha;
	}


	/**
	 * @param la fecha en formato String ss/MM/yy
	 */
	public void setFecha(String fecha) {
		DateFormat sdf = new SimpleDateFormat("dd/MM/yy");
		try {
			this.fecha = new Date(sdf.parse(fecha).getTime());            
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @return La fecha del entrenamiento
	 */
	public Date getFecha() {
		return fecha;
	}

	/**
	 * 
	 * @param id el identificador único del entrenamiento
	 */
	public void setId(long id) {
		this.id = id;
	}

	/**
	 * 
	 * @return identificador único del entrenamiento
	 */
	public long getId() {
		return id;
	}

	/**
	 * @param actividades the actividades to set
	 */
	public void setActividades(List<Actividad> actividades) {
		this.actividades = actividades;
	}

	/**
	 * @return the actividades
	 */
	public List<Actividad> getActividades() {
		return actividades;
	}

	/**
	 * @param descripcion the descripcion to set
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	/**
	 * @return the descripcion
	 */
	public String getDescripcion() {
		return descripcion;
	}
	

	/**
	 * @return los kilometros totales de todas las actividades del entrenamiento
	 */
	public float getKilometros() {
		float kilometros = 0;
		for (Object obj: this.actividades) {
			Actividad act = (Actividad) obj;
			if (act != null)
			kilometros += act.getDistancia();
		}
		return kilometros/1000;
	}
	
	/**
	 * @return el tiempo total de todas las actividades del entrenamiento en formato h:m:s.ms
	 */
	public String getTiempoTotal() {
		long tiempo = 0;
		for (Object obj: this.actividades) {
			Actividad act = (Actividad) obj;
			if (act != null)
			tiempo += act.getTiempo();
		}
		int aux = (int) (tiempo%100);
		String ms = String.valueOf(aux);
		tiempo = (long) Math.floor(tiempo/100);
		aux = (int) (tiempo%60);
		String sec = String.valueOf(aux);
		tiempo = (long) Math.floor(tiempo/60);
		aux = (int) (tiempo%60);	
		String min = String.valueOf(aux);
		tiempo = (long) Math.floor(tiempo/60);
		String hor = String.valueOf(tiempo);	
		return hor+":"+min+":"+sec+"."+ms;
	}

	/**
	 * 
	 * @return Devuelve el valor del día de la fecha del entrenamiento
	 */
	public String getDia() {
		if (this.fecha== null) return null;
		Calendar calendar = new GregorianCalendar();
		calendar.setTime((java.util.Date)this.fecha);
		return String.valueOf(calendar.get(Calendar.DAY_OF_MONTH));
	}
	
	/**
	 * 
	 * @return Devuelve el valor del mes de la fecha del entrenamiento
	 */
	public String getMes() {
		if (this.fecha== null) return null;
		Calendar calendar = new GregorianCalendar();
		calendar.setTime((java.util.Date)this.fecha);
		return String.valueOf(calendar.get(Calendar.MONTH)+1);
	}
	
	/**
	 * 
	 * @return Devuelve el valor del año de la fecha del entrenamiento
	 */
	public String getAno() {
		if (this.fecha== null) return null;
		Calendar calendar = new GregorianCalendar();
		calendar.setTime((java.util.Date)this.fecha);
		return String.valueOf(calendar.get(Calendar.YEAR));
	}

	/**
	 * 
	 * @return Devuelve los segundos totales del entrenamiento (a partir de sus actividades)
	 */
	public long getSegundosTotales() {
		long tiempo = 0;
		for (Object obj: this.actividades) {
			Actividad act = (Actividad) obj;
			if (act != null)
			tiempo += act.getTiempo();
		}
		return tiempo;
	}
}
